////////////////////////////////////////////////////////////////////// 
// fugue/HaploGraph.cpp 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the FUGUE source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile FUGUE.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Tuesday July 06, 2004
// 
 
#include "HaploGraph.h"
#include "StringArray.h"
#include "MathVector.h"
#include "Error.h"

#include <ctype.h>

void HaplotypeGraph::LoadFromFile(FILE * file, int haplos)
   {
   Dimension(haplos, markers);

   int haplo = 0;

   String input, index;
   StringArray tokens, tokens2;

   lastGenotype.Set(-1);

   while (haplo < count)
      {
      if (feof(file))
         error("Unexpected end of input file\n");

      input.ReadLine(file);
      tokens.Clear();
      tokens.AddTokens(input);

      if (tokens.Length() == 0) continue;

      if (tokens.Length() != markers + 1)
         error("Expecting %d genotypes per line, but this line is different:\n"
               "%s\n", markers, (const char *) input);

      for (int m = 0; m < markers; m++)
         if (isalpha(tokens[m+1][0]))
            {
            haplotypes[haplo][m] = 0;

            index = m;

            int j = 0;
            while (isalpha(tokens[m+1][j]))
               index += tokens[m+1][j++];

            AlleleGraph * graph = unknowns.GetGraph(index);

            int p = tokens[m+1].FastFind(',');

            graph->marker = m;
            graph->Append(haplo,
                          tokens[m+1].SubStr(j, p - j).AsInteger(),
                          tokens[m+1].SubStr(p + 1).AsInteger());

            lastGenotype[haplo] = m;
            }
         else if (tokens[m+1].AsInteger() == 0)
            {
            haplotypes[haplo][m] = 0;

            index = m;
            index += '{';
            index += haplo;
            index += '}';

            MissingAllele * missing = unknowns.GetMissing(index);

            missing->haplotype = haplo;
            missing->marker = m;
            }
         else
            haplotypes[haplo][m] = tokens[m+1].AsInteger(),
            lastGenotype[haplo] = m;

      haplo++;
      }
   }

double HaplotypeGraph::Complexity()
   {
   Vector factors;

   factors.Dimension(count);
   factors.Set(1);

   double c = 1;

   for (int i = 0; i < unknowns.Length(); i++)
      {
      Unknown * u = (Unknown *) unknowns.Object(i);

      if (u->type == U_GRAPH)
         c *= 2;
      else
         {
         MissingAllele * m = (MissingAllele *) u;

         if (lastGenotype[m->haplotype] == -1)
            continue;

         factors[m->haplotype] *= alleleCounts[m->marker];
         }
      }

   return c * factors.Sum();
   }

void HaplotypeGraph::Dimension(int haplos, int markers)
   {
   count = haplos;

   lastGenotype.Dimension(count);
   lastGenotype.Set(-1);

   haplotypes = new IntArray[count];

   for (int i = 0; i < count; i++)
      haplotypes[i].Dimension(markers);
   }

void HaplotypeGraph::UpdateAlleleCounts()
   {
   for (int i = 0; i < count; i++)
      for (int j = 0; j < markers; j++)
         if (haplotypes[i][j] > alleleCounts[j])
            alleleCounts[j] = haplotypes[i][j];

   for (int i = 0; i < unknowns.Length(); i++)
      if (unknowns.GetUnknown(i)->type == U_GRAPH)
         {
         AlleleGraph * graph = (AlleleGraph *) unknowns.GetUnknown(i);

         for (int j = 0; j < graph->alleles.Length(); j++)
            if (graph->alleles[j] > alleleCounts[graph->marker])
               alleleCounts[graph->marker] = graph->alleles[j];
         }
   }


 
