////////////////////////////////////////////////////////////////////// 
// fugue/SparseLikelihood.h 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the FUGUE source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile FUGUE.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Tuesday July 06, 2004
// 
 
#ifndef __SPARSELIKELIHOOD_H__
#define __SPARSELIKELIHOOD_H__

#include "HaploGraph.h"
#include "HaploSet.h"
#include "MathMatrix.h"
#include "HaploTree.h"

class SparseLikelihood : public MarkerInfo
   {
   public:
      int    haplos;
      Vector frequencies;

      SparseLikelihood()  { trees = NULL; index = NULL; tol = 1e-7; ztol = 1e-10; }
      ~SparseLikelihood()
         { if (index != NULL) delete [] index;
           if (trees != NULL) delete [] trees; }

      void Initialize();
      void RandomEM(HaplotypeSets * sets);
      void EM(HaplotypeSets * sets);

      void Print(double minfreq);
      void PrintExtras(double minfreq);

      double SetLikelihood(HaplotypeSet * set);
      double GraphLikelihood(HaplotypeGraph * graph);
      double GraphLikelihood(HaplotypeGraph * graph, int tree, int i);
      double CurrentLikelihood(HaplotypeGraph * graph, int tree);

      double best_llk;

      double tol, ztol;

   private:
      IntArray * index;
      IntArray last;
      IntArray new_index;
      Vector   marginals;

      bool     secondPass;
      double   setLikelihood;

      Vector   new_frequencies;

      bool UpdateIndex(HaplotypeGraph * graph, int tree, int i);
      bool ExpandIndex(HaplotypeGraph * graph, int tree, int i);
      bool SelectAllele(int tree, int i, int allele);

      void CoreEM(HaplotypeSets * sets);

      // Each tree has up to (2^treeBits) markers
      int treeBits;

      // Number of trees
      int treeCount;

      // Array of trees
      HaploTree * trees;

      // Likelihood factors within each graph
      Matrix   factorMatrix;
      Vector * factors;
   };

#endif

 
